
/****************************************************************************
 * NCSA HDF                                                                 *
 * National Comptational Science Alliance                                   *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING file.                                                        *
 *                                                                          *
 ****************************************************************************/


package ncsa.hdf.jhv;

/**
 *  This class specifies new object to hold the sds slice information
 */
public class SDSDimInfo
{
    /** the rank of the sds */
    int rank;

    /** the size of each dimension */
    int dimSize[];

    /** indices of the selected dimensions
     *  selected[0] = horizontal axis
     *  selected[1] = vertical axis
     *  selected[2] = slice axis
     */
     int selected[];

    /** starting position of the selected dimensions
     *  start[0] = horizontal axis
     *  start[1] = vertical axis
     *  start[2] = slice axis
     */
     int start[];

    /** ending position of the selected dimensions
     *  end[0] = horizontal axis
     *  end[1] = vertical axis
     *  end[2] = slice axis
     */
     int end[];

    /** current planes of the selected dimensions
     *  planes[0] = horizontal axis
     *  planes[1] = vertical axis
     *  planes[2] = slice axis
     */
     int planes[];

    /** create default SDS information object */
    public SDSDimInfo()
    {
        rank = 0;
        dimSize = null;
        selected = null;
        start = null;
        end = null;
        planes = null;
    }

    /** create SDS information object with given rank and dimension sizes*/
    public SDSDimInfo(int n, int[] sizes)
    {
        rank = n;
        dimSize = sizes;
        selected = new int[3];
        start = new int[3];
        end = new int[3];
        planes = new int[3];

        if (n>2)
        {
            setSelected(2, 1, 0);
            setStart(0, 0, 0);
            setEnd(sizes[2]-1, sizes[1]-1, sizes[0]-1);
            setPlane(0, 0, 0);
        }
    }

    /** create SDS information object with given all SDS information */
    public SDSDimInfo(int n, int[] dimSize, int[] selected,
        int[] start, int[] end, int[] planes)
    {
        this.rank = rank;
        this.dimSize = dimSize;
        this.selected = selected;
        this.start = start;
        this.end = end;
        this.planes = planes;
    }

    /** set the rank */
    public void setRank(int r)
    {
        rank = r;
    }

    /** set the dimension size */
    public void setSize(int x, int y, int z)
    {
        dimSize[0] = x;
        dimSize[1] = y;
        dimSize[2] = z;
    }

    /** set the size of a specific dimension */
    public void setSize(int index, int value)
    {
        this.dimSize[index] = value;
    }

    /** set the diemnsion with given indices*/
    public void setSelected(int x, int y, int z)
    {
        selected[0] = x;
        selected[1] = y;
        selected[2] = z;
    }

    /** set the specific diemnsion */
    public void setSelected(int index, int value)
    {
        this.selected[index] = value;
    }

    /** set the starting positions of all dimensions */
    public void setStart(int x, int y, int z)
    {
        start[0] = x;
        start[1] = y;
        start[2] = z;
    }

    /** set the starting positions of a specific diemnsion */
    public void setStart(int index, int value)
    {
        start[index] = value;
    }

    /** set the ending positions of all dimensions */
    public void setEnd(int x, int y, int z)
    {
        end[0] = x;
        end[1] = y;
        end[2] = z;
    }

    /** set the ending positions of a specific diemnsion */
    public void setEnd(int index, int value)
    {
        end[index] = value;
    }

    /** set the current planes of all dimensions */
    public void setPlane(int x, int y, int z)
    {
        planes[0] = x;
        planes[1] = y;
        planes[2] = z;
    }

    /** set the current plane of a specific diemnsion */
    public void setPlane(int index, int value)
    {
        this.planes[index] = value;
    }

    /** get the rank of SDS */
    public int getRank()
    {
        return rank;
    }

    /** get the dimension sizes */
    public int[] getSize()
    {
        return dimSize;
    }

    /** get the size of a specific dimension */
    public int getSize(int index)
    {
        return dimSize[index];
    }

    /** get the selected dimensions */
    public int[] getSelected()
    {
        return selected;
    }

    /** get the index of a specific dimension */
    public int getSelected(int index)
    {
        return selected[index];
    }

    /** get the starting positions of all dimensions */
    public int[] getStart()
    {
        return start;
    }

    /** get the starting positions of a specific diemnsion */
    public int getStart(int index)
    {
        return start[index];
    }

    /** get the ending positions of all dimensions */
    public int[] getEnd()
    {
        return end;
    }

    /** get the ending positions of a specific diemnsion */
    public int getEnd(int index)
    {
        return end[index];
    }

    /** get the current planes of all dimensions */
    public int[] getPlane()
    {
        return planes;
    }

    /** get the current plane of a specific diemnsion */
    public int getPlane(int index)
    {
        return planes[index];
    }

}
