package pupi.reader;
import pupi.reader.restart.*;
import dataspork.tree.*;
import dataspork.reader.*;
import dataspork.interp.*;
import java.io.*;
import java.net.URL;
import java.util.Enumeration;
import com.sun.xml.parser.Resolver;
import ncsa.hdf.hdflib.HDFLibrary;

public class RSDataReader implements DataReader {
  protected HDFLibrary hdf;

  public RSDataReader() { 
    hdf=new HDFLibrary();
  }

  public DataTreeNode read(URL url) {
    DataTreeNode root = null;
    try {
      RSDocHandler handler = new RSDocHandler();
      HDFParser parser = new HDFParser(hdf);

      String fileName = url.getFile();
      int k = fileName.lastIndexOf('/');
      String shortFilename = fileName.substring(++k);
      handler.setFilename(shortFilename);

      parser.setDocumentHandler(handler);
      parser.parse(fileName);

      root = handler.getDataTree();
      if (root==null) {
        root=new DataTreeNode(fileName+" unreadable");
      }
      else {
        root.addInterpreter(new InterpreterClass(SimulationInfo.class,root));
      }
    } catch (Throwable t) {t.printStackTrace ();}
    return root;
  }

  public static void main(String argv[]) throws IOException {
    DataReader reader = new RSDataReader();
    URL url = new URL("file:/home1/shumway/ex003.rs");
    DataTreeNode root = reader.read(url);
    printTree(root,"");
  }

  public static void printTree(DataTreeNode node, String indent) {
    System.out.println(indent+node.getAttribute(node.NAME));
    Enumeration a = node.getAttributeLabels();
    while (a.hasMoreElements()) {
      String label=(String)a.nextElement();
      if (!label.equals(node.NAME)) {
        System.out.println(indent+": attribute label: "+label);
      }
    }
    Enumeration e;
    if ( (e=node.children()) != null) {
      while (e.hasMoreElements()) {
        printTree( (DataTreeNode)e.nextElement(), indent+"  " );
      }
    } else {
      DataSource ds = node.getDataSource();
      String info = ds.TYPENAME[ds.getType()];
      int[] dims = ds.getDims();
      for (int i=0; i<dims.length; i++) {
        info +=  "[" + String.valueOf(dims[i]) + "]";
      }
      System.out.println(indent+": "+info);
    }
  }

}
