package pupi.reader.restart;

/** These tags are used in structured restart, packed configuration,
  *    and scalar files
  * New tags can be added, but the values of existing tags should never
  *    change.
  */
public class UpiTags{

  public static final int TR_USERBITS= 9;
  public static final int TR_BASE    = 32768;
  public static final int TR_USERMASK= 511;
  public static final int TR_SYSMASK = (~TR_USERMASK);
  public static final int TR_BRANCH  = (TR_BASE);
  public static final int TR_INT4    = (TR_BASE + 1*(1<<TR_USERBITS));
  public static final int TR_REAL8   = (TR_BASE + 2*(1<<TR_USERBITS));
  public static final int TR_STR64   = (TR_BASE + 3*(1<<TR_USERBITS));
  public static final int TR_INT4VEC = (TR_BASE + 4*(1<<TR_USERBITS));
  public static final int TR_REAL8VEC= (TR_BASE + 5*(1<<TR_USERBITS));
  public static final int TR_DATE    = (TR_BASE + 6*(1<<TR_USERBITS));

  public static int getUser(int i) {return i&TR_USERMASK;}
  public static int getSys(int i) {return i&TR_SYSMASK;}

  public static boolean isGroup(int tag)    {return getSys(tag)==TR_BRANCH;}
  public static boolean isInt4(int tag)     {return getSys(tag)==TR_INT4;}
  public static boolean isReal8(int tag)    {return getSys(tag)==TR_REAL8;}
  public static boolean isString64(int tag) {return getSys(tag)==TR_STR64;}
  public static boolean isDate(int tag)     {return getSys(tag)==TR_DATE;}
  public static boolean isVec(int tag)  {
    return ((getSys(tag)==TR_INT4VEC)||(getSys(tag)==TR_REAL8VEC));
  }


  static final int UPI_RS_MAGIC=2000;

  static final int UPI_GROUP=-1;
  static final int UPI_END=0;
  static final int UPI_LABEL=1;
  static final int UPI_EPOCH=2;
  static final int UPI_TIME=3;
  static final int UPI_RNG=4;
  static final int UPI_LCA=5;
  static final int UPI_VM=6;
  static final int UPI_NODE=7;
  static final int UPI_PATH=8;
  static final int UPI_SEGMENT=9;
  static final int UPI_ORIGIN=10;
  static final int UPI_SLICES=11;
  static final int UPI_PARTICLES=12;
  static final int UPI_PERM=13;
  static final int UPI_REF1=14;
  static final int UPI_REF2=15;
  static final int UPI_JOIN=16;
  static final int UPI_COORDS=17;
  static final int UPI_CALC=18;
  static final int UPI_STATES=19;
  static final int UPI_DIM=20;
  static final int UPI_STATE=21;
  static final int UPI_HBS2M=22;
  static final int UPI_SPECIES=23;
  static final int UPI_BOX=24;
  static final int UPI_ARCH=25;
  static final int UPI_HOSTNAME=26;
  static final int UPI_INVPERM=27;
  static final int UPI_SETS=28;
  static final int UPI_BLOCKS=29;
  static final int UPI_INT4=30;
  static final int UPI_INT4VEC=31;
  static final int UPI_REAL8=32;
  static final int UPI_REAL8VEC=33;
  static final int UPI_STR64=34;
  static final int UPI_STR64VEC=35;
  static final int UPI_STRUCT=36;
  static final int UPI_INDEX=37;
  static final int UPI_RESTART=38;
  static final int UPI_BLOCKAVG=39;
  static final int UPI_BLOCKSUM=40;
  static final int UPI_BLOCKNORM=41;
  static final int UPI_SCALARS=42;
  static final int UPI_SCALAR=43;
  static final int UPI_OBSERVABLES=44;
  static final int UPI_UNITS=45;
  static final int UPI_SCSTATS=46;
  static final int UPI_VECTORS=47;
  static final int UPI_GR=48;
  static final int UPI_DATA=49;
  static final int UPI_ERROR=50;
  static final int UPI_SKPOINTS=51;
  static final int UPI_SK=52;
  static final int UPI_EVIRIAL=53;
  static final int UPI_GLOBAL_RNG=54;

  static final String[] names = {
  "end","label","epoch","time","rng",
  "lca","vm","node","path","segment",
  "origin","slices","particles","perm","ref1",
  "ref2","join","coords","calc","states",
  "dim","state","hbs2m","species","box",
  "arch","hostname","invperm","sets","blocks",
  "int4","int4vec","real8","real8vec","str64",
  "str64vec","struct","index","restart","blockavg",
  "blocksum","blocknorm","scalars","scalar","observables",
  "units","scstats","vectors","gr","data",
  "error","skpoints","sk","evirial","global-rng"
  };

  public static String name(int tag) {
    tag = getUser(tag);
    if (tag>=0 && tag<names.length) {
      return names[tag];
    }else{
      return "unknown"+tag;
    }
  }

  public static String getString(int fileID, int tag, int ref) {
    return nGetString(fileID, tag, ref);
  }

  public static native String nGetString(int fileID, int tag, int ref);

}
